import json
import logging
import os
import threading
import time

from src.api.Aria2API import Aria2API
from src.api.alist import AlistAPI
from src.api.radarr import RadarClient


class RemoteDownloadTask:
    def __init__(self, config, parent_dir):

        self.current_downloads = None
        self.completed_downloads = None
        self.config = config
        self.is_running = False

        self.monitor_thread = None
        self.download_thread = None
        self.check_interval = int(config['COMMON']['CHECK_INTERVAL'])
        self.remote_alist_api = AlistAPI(self.config['REMOTE_ALIST']['API_URL'],
                                         self.config['REMOTE_ALIST']['USERNAME'],
                                         self.config['REMOTE_ALIST']['PASSWORD'])

        self.radar_client = RadarClient(self.config['RADAR']['URL'],
                                        self.config['RADAR']['API_KEY'])

        self.home_alist_api = AlistAPI(self.config['HOME_ALIST']['API_URL'],
                                       self.config['HOME_ALIST']['USERNAME'],
                                       self.config['HOME_ALIST']['PASSWORD'])

        self.aria2_api = Aria2API(config['ARIA2']['RPC_URL'], config['ARIA2']['RPC_SECRET'])

        # 读取路径等配置
        self.remote_alist_download_path = self.config['REMOTE_ALIST']['DOWNLOAD_PATH']
        self.home_alist_download_path = self.config['HOME_ALIST']['DOWNLOAD_PATH']
        self.home_scy_alist_copy_path = self.config['HOME_ALIST']['SCY_COPY_PATH']
        self.home_des_alist_copy_path = self.config['HOME_ALIST']['DES_COPY_PATH']
        self.remote_data = self.config['COMMON']['REMOTE_DATA']
        self.home_data = self.config['COMMON']['HOME_DATA']
        self.aria2_download_path = self.config['ARIA2']['DESTINATION_PATH']
        self.aria2_docker_download_path = self.config['ARIA2']['DOCKER_DOWNLOAD_PATH']
        self.parent_dir = parent_dir
        self.home_json_path = os.path.join(parent_dir, 'data', self.home_data)




    def start_monitoring(self):
        if not self.is_running:
            self.is_running = True
            self.monitor_thread = threading.Thread(target=self.monitor_aria2_and_delete)
            self.download_thread = threading.Thread(target=self.process_downloads)

            self.monitor_thread.start()
            self.download_thread.start()

    def stop_monitoring(self):
        self.is_running = False
        if self.monitor_thread:
            self.monitor_thread.join()
        if self.download_thread:
            self.download_thread.join()

    def _save_directory_contents(self,
                                 remote_download_path,
                                 local_download_path,
                                 scy_copy_path,
                                 des_copy_path,
                                 parent_dir):
        """获取远程和本地对应的目录结构，并保存为 JSON 文件"""
        # 获取远程目录结构
        remote_data = self.remote_alist_api.recursive_collect_contents(
            remote_download_path,
            local_download_path
        )
        remote_json_path = os.path.join(parent_dir, 'data', 'remote_data.json')
        with open(remote_json_path, 'w', encoding='utf-8') as f:
            json.dump(remote_data, f, indent=4)

        # # 获取本地目录结构
        # home_data = self.home_alist_api.recursive_collect_contents(
        #     scy_copy_path, des_copy_path, scy_copy_path, des_copy_path
        # )
        # home_json_path = os.path.join(parent_dir, 'data', self.home_data)
        # with open(home_json_path, 'w', encoding='utf-8') as f:
        #     json.dump(home_data, f, indent=4)

        return remote_data

    def init_download_states(self):
        # 获取已完成下载的电影文件名
        self.completed_downloads = self.radar_client.get_all_movie_names()

        # 获取当前 Aria2 下载队列中的文件名
        self.current_downloads = {os.path.basename(file.path) for download in self.aria2_api.get_downloads() for file in
                                  download.files if file.selected}

    def process_downloads(self):
        logging.info('Start remote download monitoring')
        self.init_download_states()
        while self.is_running:
            # 更新当前下载队列集合
            self.current_downloads = {os.path.basename(file.path) for download in self.aria2_api.get_downloads() for
                                      file in download.files if file.selected}

            # 获取已完成电影的文件名集合
            already_movie_names = self.radar_client.get_all_movie_names()

            for home in already_movie_names:
                if home not in self.current_downloads and home not in self.current_downloads:
                    # 获取远程目录结构
                    remote_json_path = self._save_directory_contents(self.remote_alist_download_path,
                                                                     self.home_alist_download_path,
                                                                     self.home_scy_alist_copy_path,
                                                                     self.home_des_alist_copy_path, self.parent_dir)

                    for home_item in remote_json_path:
                        if not home_item['is_dir'] and home_item['name'] == home:
                            cent_data = self.remote_alist_api.get_file_or_directory_info(home_item['path'])
                            if cent_data:
                                logging.info(f"name {home_item['name']}")
                                # self.send_to_aria2(cent_data['data']['raw_url'])
                                logging.info(f"Started to download {cent_data['data']['raw_url']}")
                                self.current_downloads.add(home_item['name'])


                            else:
                                logging.info("File not found")

            time.sleep(self.check_interval)

    def send_to_aria2(self, file_path):
        # 将文件添加到 Aria2 下载队列
        self.aria2_api.add_url(file_path, options={"dir": self.aria2_docker_download_path + "/movie"})
        logging.info(f"Added to Aria2: {file_path}")

    def monitor_aria2_and_delete(self):
        logging.info('Start remote Aria2 download is_complete monitoring')
        while self.is_running:
            # 获取 Aria2 当前的下载列表
            downloads = self.aria2_api.get_downloads()
            for download in downloads:
                if download.is_complete:
                    pass
                    # movie_name = self.radar_client.get_all_movie_names()
                    # for file_name in movie_name:
                    #     # logging.info(f"Download completed: {file_name}")
                    #     # 处理每个下载完成的文件
                    #     # 调用方法并获取返回的电影 ID
                    #     movie_id = self.radar_client.find_movie_id_by_filename(file_name)
                    #
                    #     # 打印结果
                    #     if movie_id is not None:
                    #         print(f"Found movie ID for '{file_name}': {movie_id}")
                    #     else:
                    #         print(f"No movie found for '{file_name}'")

            time.sleep(self.check_interval)
