import json
import logging
import os
import threading
import time

from src.api.QbittorrentManager import QBittorrentManager


class RemoteQbTask:
    def __init__(self, config, parent_dir,socketio):

        self.is_running = False
        self.threads = []
        self.parent_dir = parent_dir
        self.qb_manager = QBittorrentManager(config['qbittorrent']['URL'], config['qbittorrent']['USERNAME'],
                                             config['qbittorrent']['PASSWORD'])
        self.socketio = socketio

    def start_monitoring(self):
        if not self.is_running:
            self.is_running = True
            downloading_torrent_hashes = self.get_downloading_torrent_hashes()

            for torrent_hash in downloading_torrent_hashes:
                self.start_monitoring_thread(torrent_hash)

    def stop_monitoring(self):
        self.is_running = False
        for thread in self.threads:
            if thread.is_alive():
                thread.join()

    def login(self):
        self.qb_manager.login()

    def get_torrent_hashes(self):
        """获取当前所有torrents的哈希值列表"""
        return [torrent['hash'] for torrent in self.qb_manager.get_torrents()]

    def get_downloading_torrent_hashes(self):
        """获取所有正在下载的torrents的哈希值列表"""
        torrents = self.qb_manager.get_torrent_filters(filter='downloading')
        return [torrent['hash'] for torrent in torrents]

    def send_to_qb(self, torrent):
        """添加torrent并获取新添加的torrent的哈希值，然后监控下载"""
        before_hashes = set(self.get_torrent_hashes())
        self.qb_manager.add_torrent(torrent)
        time.sleep(5)  # 等待torrent出现

        after_hashes = set(self.get_torrent_hashes())
        new_hashes = after_hashes - before_hashes

        for torrent_hash in new_hashes:
            self.start_monitoring_thread(torrent_hash)

        return list(new_hashes)

    def check_download_complete(self, torrent_hash):
        """检查指定的torrent是否下载完成"""
        torrent_info = self.qb_manager.get_torrents_hash(torrent_hash)
        return torrent_info['pieces_have'] == torrent_info['pieces_num']

    def start_monitoring_thread(self, torrent_hash):
        """为给定的torrent哈希值启动一个新的监控线程"""
        thread = threading.Thread(target=self.monitor_download, args=(torrent_hash,))
        self.threads.append(thread)
        thread.start()

    def monitor_download(self, torrent_hash):
        """监控指定torrent的下载状态，长时间无活动则删除"""
        logging.info(f"开始监控Torrent {torrent_hash}")
        inactive_threshold = 3600  # 设置无活动阈值，例如3600秒
        last_activity_time = time.time()  # 初始化最后活动时间
        while self.is_running:
            torrent_info = self.qb_manager.get_torrents_hash(torrent_hash)
            logging.info(f"Torrent {torrent_info}")

            torrent_info_file = self.qb_manager.get_torrents_hash_file(torrent_hash)
            logging.info(f"Torrent {torrent_info_file}")
            # 检查下载是否完成
            if self.check_download_complete(torrent_hash):
                logging.info(f"Torrent {torrent_hash} 下载完成")
                break
            # 检查torrent是否仍在活动
            if torrent_info['dl_speed'] > 0 or torrent_info['up_speed'] > 0:
                last_activity_time = time.time()  # 更新最后活动时间
            else:
                # 检查自上次活动以来是否超过了无活动阈值
                if time.time() - last_activity_time > inactive_threshold:
                    logging.info(f"Torrent {torrent_hash} 长时间无活动，正在删除...")
                    self.qb_manager.delete_torrent(torrent_hash)
                    break
            logging.info(f"Torrent {torrent_hash} 下载中...")

            download_speed = torrent_info['dl_speed']  # 下载速度，单位bytes/s
            upload_speed = torrent_info['up_speed']  # 上传速度，单位bytes/s
            # 转换速度为更易读的格式（如，KB/s 或 MB/s）
            download_speed_kbps = download_speed / 1024  # 转换为KB/s
            upload_speed_kbps = upload_speed / 1024  # 转换为KB/s

            # 假设列表的第一个文件是我们关心的主文件
            torrent_name = torrent_info_file[0].get('name', 'Unknown') if torrent_info_file else 'Unknown'

            # 使用socketio发送实时信息
            torrent_status = {
                'name': torrent_name,
                'download_speed': round(download_speed_kbps, 2),
                'upload_speed': round(upload_speed_kbps, 2),
                'status': 'Downloading'
            }
            self.socketio.emit('torrents_status_update', {'torrents_status_update': torrent_status})

            time.sleep(5)  # 每1秒检查一次
